#include <math.h>
#include <iostream>
#include <assert.h>
#include <iomanip>

#include "TvcDriver.hh"

using namespace std;
const unsigned NUM_REGISTERS = 14;
const unsigned STATUS_REGISTER = 13;
const unsigned COMMAND_REGISTER = 12;

TvcDriver::TvcDriver(EPPPort &rTPort) :
  rPort(rTPort)
{}

TvcDriver::~TvcDriver(void)
{}

void TvcDriver::DumpRegisters(void)
{
//   for(unsigned ii = 0; ii < 7; ii++)
//     {
//       rPort.WriteAddress(ii);

//       unsigned char value = 0;
//       rPort.ReadData(value);
//       cout << "reg ii: " << ii << " " << setw(2) << setfill('0') << hex << (int)value << dec << endl;
//     };
  unsigned char values[NUM_REGISTERS];
  for(unsigned ii = 0; ii < NUM_REGISTERS; ii++)
    {
      rPort.WriteAddress(ii);

      rPort.ReadData(values[ii]);
      cout << "reg ii: " << ii << " " << setw(2) << setfill('0') << hex << (int)values[ii] << dec << endl;
    };

  unsigned long address = ((unsigned long)values[0])<<16 | ((unsigned long)values[1])<<8 | ((unsigned long)values[2]);
  cout  << setw(8) << setfill('0') << hex << address << dec << " " << address << endl;
};

void TvcDriver::ClearRegisters(void)
{
  for(unsigned ii = 0; ii < NUM_REGISTERS-2; ii++) //minus two to not write to the command or status register
    {
      rPort.WriteAddress(ii);
      rPort.WriteData(0x00);
    };
}

void TvcDriver::PrintStatus(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  unsigned char status = 0;
  rPort.ReadData( status );
 
  if( status & 0x80 )
    cout << "epp error bit set" << endl;
  else
    cout << "epp error bit clear" << endl;

  if( status & 0x40 )
     cout << "block reader idle" << endl;
  else
    cout << "block reader busy" << endl;

  if( status & 0x20 )
     cout << "block writer idle" << endl;
  else
    cout << "block writer busy" << endl;

  if( status & 0x10 )
     cout << "pix writer idle" << endl;
  else
    cout << "pix writer busy" << endl;

}

/*
  this was a quicky test case in main that I though might be useful (with changes) to test the 
  framebuffer.  I just stuck the code here where I might find it later.
*/
void TvcDriver::TestRam(void)
{
  const unsigned write_count = 255;
  const unsigned read_count = 255;

  unsigned long bad_count = 0;
  unsigned long presumed_bad_writes = 0;

  for( unsigned ii = 0; ii < write_count; ii++)
    {
      unsigned long dataValue = (unsigned long)rint( (((double)random())/((double)RAND_MAX))* 0xFFFFFFFF);
      cout << "storing value: " << hex << dataValue << dec << endl;
      
      WriteValue(0x00, dataValue);

      unsigned bad_read_count = 0;
      for(unsigned long jj = 0; jj < read_count; jj++)
	{
	  ClearRegisters();
	  
	  unsigned long result = 0;
	  ReadValue(0, result);
	  //driver.DumpRegisters();
	  if( result != dataValue )
	    {
	      cout << setfill('0') << dec << setw(5) << jj
		   << "\t error stored != read stored: " << setfill('0') << setw(8) << hex << dataValue
		   << " result: " << setfill('0') << setw(8) << hex << result << dec << endl;
	      bad_read_count++;
	    };
	};
      bad_count += bad_read_count;

      if(bad_read_count > read_count/2)
	presumed_bad_writes++;
    };

  cout << "bad count: " << bad_count << endl;
  cout << "error factor: " << setprecision(5) << (double)bad_count/(double)(write_count*read_count) << endl;
  cout << "factor that could be due to bad writes: " << setprecision(5)
       << (double)(presumed_bad_writes*write_count)/(double)(write_count*read_count) << endl;
}


void TvcDriver::Blank(unsigned long value)
{
  for(unsigned long ii = 0; ii < 640*480/4; ii++)
    WriteValue(ii*4 , value);
}

void TvcDriver::DrawPixel(int xPos, int yPos, unsigned char pixValue)
{
  WriteFBAddress( xPos + yPos*640 );

  rPort.WriteAddress(4);
  rPort.WriteData( pixValue );

  rPort.WriteAddress(COMMAND_REGISTER);
  rPort.WriteData( 0x03 );

  WaitOnPW();
}

void TvcDriver::DrawLine(int xStart, int yStart, int xEnd, int yEnd,
			 unsigned char pixValue)
{
  int deltaX = xEnd - xStart;
  int deltaY = yEnd - yStart;
  bool xRev = false;
  bool yRev = false;

  if(deltaX < 0)
    {
      xRev = true;
      deltaX *= -1;
    };

  if(deltaY < 0)
    {
      yRev = true;
      deltaY *= -1;
    };

  int lXStart = xStart;
  int lXEnd = xEnd;
  int lYStart = yStart;
  int lYEnd = yEnd;

  if(deltaY > deltaX)
    {
      cout << "y dominant line" << endl;
      if(yRev)
	{
	  lXStart = xEnd;
	  lXEnd = xStart;
	  lYStart = yEnd;
	  lYEnd = yStart;
	  deltaX *= -1;
	};

      for(int cY = lYStart; cY <= lYEnd; cY++)
	{
	  int xVal = lXStart +
	    (int)rint( ((float)deltaX)*((float)(cY - lYStart))/((float)deltaY) );
	  DrawPixel(xVal, cY, pixValue);
	  cout << xVal << " " << cY << endl;
	};
    }else
      {
	cout << "x dominant line" << endl;
	if(xRev)
	  {
	    lXStart = xEnd;
	    lXEnd = xStart;
	    lYStart = yEnd;
	    lYEnd = yStart;
	    deltaY *= -1;
	  };

	for(int cX = lXStart; cX < lXEnd; cX++)
	  {
	    int yVal = lYStart +
	      (int)rint( ((float)deltaY)*((float)(cX - lXStart))/((float)deltaX) );
	    DrawPixel(cX, yVal, pixValue);
	    cout << cX << " " << yVal << endl;
	  };
      };
}

void TvcDriver::ReadValue(unsigned long fbAddress, unsigned long &rValue)
{
  WriteFBAddress(fbAddress);

  rPort.WriteAddress(COMMAND_REGISTER);
  rPort.WriteData( 0xF0 );

  WaitOnBR();

  rPort.WriteAddress(4);
  unsigned char byte3 = 0;
  rPort.ReadData(byte3);

  rPort.WriteAddress(5);
  unsigned char byte2 = 0;
  rPort.ReadData(byte2);

  rPort.WriteAddress(6);
  unsigned char byte1 = 0;
  rPort.ReadData(byte1);

  rPort.WriteAddress(7);
  unsigned char byte0 = 0;
  rPort.ReadData(byte0);

  rValue = ((unsigned long)byte3)<<24 | ((unsigned long)byte2)<<16 |
    ((unsigned long)byte1)<<8 | (unsigned long)byte0;
}

void TvcDriver::WriteValue(unsigned long fbAddress, unsigned long newValue)
{
  WriteFBAddress(fbAddress);

  rPort.WriteAddress(4);
  rPort.WriteData( (newValue&0xFF000000)>>24 );

  rPort.WriteAddress(5);
  rPort.WriteData( (newValue&0x00FF0000)>>16 );

  rPort.WriteAddress(6);
  rPort.WriteData( (newValue&0x0000FF00)>>8 );

  rPort.WriteAddress(7);
  rPort.WriteData( newValue&0x000000FF );

  rPort.WriteAddress(COMMAND_REGISTER);
  rPort.WriteData( 0x0F );

  WaitOnBW();
}


void TvcDriver::WriteFBAddress(unsigned long fbAddress)
{
  rPort.WriteAddress(0);
  rPort.WriteData( (fbAddress & 0xFF000000)>>24 );

  rPort.WriteAddress(1);
  rPort.WriteData( (fbAddress & 0x00FF0000)>>16 );

  rPort.WriteAddress(2);
  rPort.WriteData( (fbAddress & 0x0000FF00)>>8 );

  rPort.WriteAddress(3);
  rPort.WriteData( (fbAddress & 0x000000FF) );
}

void TvcDriver::WaitOnBW(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 10; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x20 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnBW" << endl;
  assert(false);
}

void TvcDriver::WaitOnBR(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 10; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x40 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnBR" << endl;
  assert(false);
}

void TvcDriver::WaitOnPW(void)
{
  rPort.WriteAddress(STATUS_REGISTER);

  for(unsigned ii = 0; ii < 10; ii++)
    {
      unsigned char status = 0;
      rPort.ReadData( status );
      if( status & 0x10 ) return;
    }

  cerr << "timeout in TvcDriver::WaitOnPW" << endl;
  assert(false);
}
