library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity eppcu is
    Port (
 --the eppcu's datalines that run to the parallel port
    		 epp_data 		: inout	std_logic_vector(7 downto 0);
           epp_write 		: in 	std_logic;
           epp_wait 		: out 	std_logic;
           epp_data_strobe 	: in 	std_logic;
           epp_address_strobe : in 	std_logic;
		 epp_irq			: out	std_logic; --not used but set to zero
--           epp_reset 		: in 	std_logic;

--the eppcu's locic interface to the rest of the tvc		 
		 tvc_clock		: in		std_logic;
		 tvc_reset		: in 	std_logic;

--interface to the block_reader memory user (mu)
		br_fb_address 		: out std_logic_vector(31 downto 0);
		br_fb_data		: in std_logic_vector(31 downto 0);
		br_start_strobe	: out std_logic;
		br_idle			: in std_logic;

--interface to the block_writer memory user (mu)
		bw_fb_address 		: out std_logic_vector(31 downto 0);
		bw_fb_data		: out std_logic_vector(31 downto 0);
		bw_start_strobe	: out std_logic;
		bw_idle			: in std_logic;

--interface to the rp_writer memory user
		rpw_fb_address 	: out std_logic_vector(31 downto 0);
		rpw_pix_data		: out std_logic_vector(7 downto 0);
		rpw_start_strobe	: out std_logic;
		rpw_idle			: in std_logic

		 --eppcu_address_0	: out	std_logic_vector(7 downto 0);
		 --eppcu_address_1	: out	std_logic_vector(7 downto 0);
		 --eppcu_adderss_2	: out	std_logic_vector(7 downto 0);
		 --eppcu_pix_value	: inout	std_logic_vector(7 downto 0)
		);	 
end eppcu;

architecture Behavioral of eppcu is
--we have four epp addresses that matter to us.
-- 0 fb-address 0
-- 1 fb-address 1
-- 2 fb-address 2
-- 3 a pixel value
-- 4 a control byte -> unstored. acted on immediatly.
--the epp host (computer) writes an address, then next writes a data byte to go into the desired address's register
	signal epp_address : STD_LOGIC_VECTOR(7 downto 0);

	signal latched_epp_write : std_logic;
	signal latched_epp_data_strobe : std_logic;
	signal latched_epp_address_strobe : std_logic;
	signal epp_error : std_logic;
	signal waiting_on_br : std_logic; --when the pr goes idle put its pr_fb_data into fb_pix registers

--registers that hold data for read/write from the epp port
	signal epp_dr_0 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_1 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_2 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_3 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_4 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_5 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_6 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_7 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_8 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_9 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_10 : STD_LOGIC_VECTOR(7 downto 0);
	signal epp_dr_11 : STD_LOGIC_VECTOR(7 downto 0);

-- we use the value for this the momemnt a write to the desired address is made. no need to store it
--	signal eppcu_command_byte : STD_LOGIC_VECTOR(7 downto 0);
 	signal wait_timer : std_logic_vector(11 downto 0);
	--only significant when sending a command to a mu. (ie. a epp data write to address 7 was made)
	type EPP_INT_STATE is ( EPP_NORMAL, EPP_DELAY_HOST_ADDRESS_WRITE_START, EPP_DELAY_HOST_DATA_WRITE_START, EPP_DELAY_NOTIFY_HOST_DATA_READ, EPP_ENDING_HOST_DATA_READ, EPP_ENDING_HOST_DATA_WRITE, EPP_ENDING_HOST_ADDRESS_WRITE, EPP_STARTING_PR, EPP_STARTING_PW, EPP_STARTING_RPW);
	signal epp_state : EPP_INT_STATE;
begin
	process(tvc_clock)
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		epp_error <= '0';
		waiting_on_br <= '0';
		epp_address <= "00000000";
		epp_dr_0 <= X"00";
		epp_dr_1 <= X"00";
		epp_dr_2 <= X"00";
		epp_dr_3 <= X"00";
		epp_dr_4 <= X"00";		
		epp_dr_5 <= X"00";		
		epp_dr_6 <= X"00";		
		epp_dr_7 <= X"00";
		epp_dr_8 <= X"00";
		epp_dr_9 <= X"00";
		epp_dr_10 <= X"00";
		epp_dr_11 <= X"00";
		br_fb_address 	<= X"0000_0000";
		epp_data <= (others => 'Z');
		epp_wait <= '0';
		br_start_strobe <= '0';
		bw_fb_address 	<= X"0000_0000";
		bw_fb_data <= X"0000_0000";
		bw_start_strobe <= '0';
		rpw_fb_address <= X"0000_0000";
		rpw_pix_data <= X"00";
		rpw_start_strobe <= '0';
		epp_state <= EPP_NORMAL;
		epp_irq <= '0';		
	else	--not a reset
	 latched_epp_write <= epp_write;
      latched_epp_data_strobe <= epp_data_strobe; 
      latched_epp_address_strobe <= epp_address_strobe;
	case epp_state is
	 when EPP_ENDING_HOST_DATA_READ => 
		if wait_timer = X"1f4" then  --a ?hackish? timeout
			epp_data <= (others=> 'Z');
			epp_state <= EPP_NORMAL;
			epp_wait <= '0';
		elsif wait_timer >= X"028" and latched_epp_data_strobe = '1' then
			epp_data <= (others => 'Z');
			epp_state <= EPP_NORMAL;
			epp_wait <= '0';
		else
			wait_timer <= wait_timer + 1;
		end if;
	when EPP_ENDING_HOST_DATA_WRITE =>
		if wait_timer = X"1f4" then --timeout
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		elsif wait_timer >= X"028" and latched_epp_data_strobe = '1' then
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		else
			wait_timer <= wait_timer + 1;
		end if;
	when EPP_ENDING_HOST_ADDRESS_WRITE =>
		if wait_timer = X"1f4" then --timeout
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		elsif wait_timer >= X"028" and latched_epp_address_strobe = '1' then
			epp_wait <= '0';
			epp_state <= EPP_NORMAL;
			epp_data <= (others => 'Z');
		else
			wait_timer <= wait_timer + 1;
		end if;
	when EPP_NORMAL =>
		if waiting_on_br = '1' and br_idle = '1' then
	    	epp_dr_4 <= br_fb_data(31 downto 24);
		epp_dr_5 <= br_fb_data(23 downto 16);
		epp_dr_6 <= br_fb_data(15 downto 8);
		epp_dr_7 <= br_fb_data(7 downto 0);
		waiting_on_br <= '0';
		end if;

		if latched_epp_address_strobe = '0' and latched_epp_write = '0' then
			wait_timer <= X"000";
			epp_state <= EPP_DELAY_HOST_ADDRESS_WRITE_START;
		elsif latched_epp_data_strobe = '0' and latched_epp_write = '0' then
			wait_timer <= X"000";
			epp_state <= EPP_DELAY_HOST_DATA_WRITE_START;
		elsif latched_epp_data_strobe = '0' and latched_epp_write = '1' then
		   case epp_address is
				when X"00" =>
					epp_data <= epp_dr_0;
				when X"01" =>
					epp_data <= epp_dr_1;
				when X"02" =>
					epp_data <= epp_dr_2;
				when X"03" =>
					epp_data <= epp_dr_3;
				when X"04" =>
					epp_data <= epp_dr_4;
				when X"05" =>
					epp_data <= epp_dr_5;
				when X"06" =>
					epp_data <= epp_dr_6;
				when X"07" =>
					epp_data <= epp_dr_7;
				when X"08" =>
					epp_data <= epp_dr_8;
				when X"09" =>
					epp_data <= epp_dr_9;
				when X"0a" =>
					epp_data <= epp_dr_10;
				when X"0b" =>
					epp_data <= epp_dr_11;
				when X"0d" =>
					epp_data <= epp_error & br_idle & bw_idle & rpw_idle & "0000";
				when others =>
					epp_data <= X"00";
			end case;
			wait_timer <= X"000";
			epp_state <= EPP_DELAY_NOTIFY_HOST_DATA_READ;
		end if; --done read and write
	when EPP_DELAY_HOST_ADDRESS_WRITE_START	=>
		if wait_timer = X"040" then
	  		epp_address <= epp_data;
			epp_wait <= '1';
			wait_timer <= X"000";
			epp_state <= EPP_ENDING_HOST_ADDRESS_WRITE;
		else
			wait_timer <= wait_timer + 1;
		end if;
	when	EPP_DELAY_HOST_DATA_WRITE_START =>
		if wait_timer = X"040" then
			case epp_address is
			when X"00" =>
				epp_dr_0 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"01" =>
				epp_dr_1 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"02" =>
				epp_dr_2 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"03" =>
				epp_dr_3 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"04" =>
				epp_dr_4 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"05" =>
				epp_dr_5 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"06" =>
				epp_dr_6 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"07" =>
				epp_dr_7 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"08" =>
				epp_dr_8 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"09" =>
				epp_dr_9 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"0a" =>
				epp_dr_10 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"0b" =>
				epp_dr_11 <= epp_data;
				epp_state <= EPP_ENDING_HOST_DATA_WRITE;
				wait_timer <= X"000";
				epp_wait <= '1';
			when X"0c" =>
				if epp_data = X"F0" then
					if br_idle = '1' then
						wait_timer <= X"000";
				 		br_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
						br_start_strobe <= '1';
						epp_state	<= EPP_STARTING_PR;
						--we cannot set this yet. as it will take a cycle for the pr reader to become non-idle. set it it STARTING_PR
					else
						epp_error <= '1';
						epp_state <= EPP_ENDING_HOST_DATA_WRITE;
						wait_timer <= X"000";
						epp_wait <= '1';
					end if;
				elsif epp_data = X"0F" then
					if bw_idle = '1' then
						bw_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
						wait_timer <= X"000";
						bw_fb_data <= epp_dr_4 & epp_dr_5 & epp_dr_6 & epp_dr_7;
						bw_start_strobe <= '1';
						epp_state <= EPP_STARTING_PW;
					else
						epp_error <= '1';
						epp_state <= EPP_ENDING_HOST_DATA_WRITE;
						wait_timer <= X"000";
						epp_wait <= '1';
					end if;
				elsif epp_data = X"03" then
					if rpw_idle = '1' then
						rpw_fb_address <=  epp_dr_0 & epp_dr_1 & epp_dr_2 & epp_dr_3;
						wait_timer <= X"000";
						rpw_pix_data <= epp_dr_4;
						rpw_start_strobe <= '1';
						epp_state <= EPP_STARTING_RPW;
					else
						epp_error <= '1';
						epp_state <= EPP_ENDING_HOST_DATA_WRITE;
						wait_timer <= X"000";
						epp_wait <= '1';
					end if;				
				else
			  		--we don't know what command was being sent, but we need to finish the cycle cleanly
					--we could flag an error.
					epp_state <= EPP_ENDING_HOST_DATA_WRITE;
					wait_timer <= X"000";
					epp_wait <= '1';
				end if;
			when others => --should never happen (invalid address in address register)
				epp_state <= EPP_ENDING_HOST_DATA_WRITE; --so we finish cleanly.
				wait_timer <= X"000";
				epp_wait <= '1';
			end case;
		else
			wait_timer <= wait_timer + 1;
		end if;
	when EPP_DELAY_NOTIFY_HOST_DATA_READ=>
		if wait_timer = X"018" then
			epp_wait <= '1';
			wait_timer <= X"000";
			epp_state <= EPP_ENDING_HOST_DATA_READ;
		else
			wait_timer <= wait_timer + 1;
		end if;
	when EPP_STARTING_PR =>
		epp_data <= (others => 'Z');
		waiting_on_br <= '1';
		br_start_strobe <= '0';
		epp_wait <= '1';
		wait_timer <= X"000";
		epp_state <= EPP_ENDING_HOST_DATA_WRITE;
	when EPP_STARTING_PW =>
		epp_data <= (others => 'Z');
		bw_start_strobe <= '0';
		epp_wait <= '1';
		wait_timer <= X"000";
		epp_state <= EPP_ENDING_HOST_DATA_WRITE;
	when EPP_STARTING_RPW =>
		epp_data <= (others => 'Z');
		rpw_start_strobe <= '0';
		epp_wait <= '1';
		wait_timer <= X"000";
		epp_state <= EPP_ENDING_HOST_DATA_WRITE;
	end case;
  	end if; --reset
	end if; --clock
	end process;
end Behavioral;
