library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity rp_writer is
	Port(
		--memory user access buss 
		mu_buss_address 	: in std_logic_vector(11 downto 0);
		mu_buss_data		: inout std_logic_vector(31 downto 0);
		mu_buss_write		: in  std_logic; --if high the MCU is going to write to a mu
		 --if high the MCU expects the selected MU to present its data on the mu_buss_data or to
		 --copy data from the mu_buss_data into its internal cache/registers.
		mu_buss_strobe			: in  std_logic;
		
		rp_writer_irq		: out std_logic;
		rp_writer_select	: in std_logic;

		--blah		 
		 tvc_clock		: in		std_logic;
		 tvc_reset		: in 	std_logic;

		 --interface to the eppcu
		epp_fb_address 	: in std_logic_vector(31 downto 0);
		epp_pix_data		: in std_logic_vector(7 downto 0);
		epp_start_strobe	: in std_logic;
		epp_idle			: out std_logic
	  );
end rp_writer;

architecture Behavioral of rp_writer is

	type RPW_STATE_INT is (RPW_IDLE, RPW_MCU_DRIVEN_READ, RPW_MCU_DRIVEN_WRITE, RPW_COMPLETING);
	signal rpw_state : RPW_STATE_INT;

 	signal fb_address : std_logic_vector(31 downto 0);
	signal pix_data : std_logic_vector(7 downto 0);

	signal mcu_selected_once : std_logic;

	signal aligned_read_value : std_logic_vector(31 downto 0);

	signal mu_buss_data_in : std_logic_vector(31 downto 0);
	signal mu_buss_data_out : std_logic_vector(31 downto 0);
	
	procedure HandleMCUDrivenReadClock(signal rpw_state 			: inout RPW_STATE_INT;
								signal fb_address 			: in std_logic_vector(31 downto 0);
								signal mcu_selected_once 	: inout std_logic;
								signal aligned_read_value 	: out std_logic_vector(31 downto 0);
								signal mu_buss_data_in		: in std_logic_vector(31 downto 0);
								signal mu_buss_data_out		: out std_logic_vector(31 downto 0);
								signal rp_writer_select 		: in std_logic ) is
	begin
		if mcu_selected_once = '1' and rp_writer_select = '0' then
			rpw_state <= RPW_MCU_DRIVEN_WRITE;
			mcu_selected_once <= '0';
			--we leave the irq line high.
		elsif rp_writer_select = '1' then
			mcu_selected_once <= '1';
			if mu_buss_address = X"FFC" then
				mu_buss_data_out <=  X"00" & X"000004";
			elsif mu_buss_address = X"FFD" then
				mu_buss_data_out <= fb_address(31 downto 2) & "00";
			elsif mu_buss_address = X"000" then
				aligned_read_value <= mu_buss_data_in;
			end if;
		end if;
	end HandleMCUDrivenReadClock;

	procedure HandleMCUDrivenWriteClock(signal rpw_state 			: inout RPW_STATE_INT;
								signal fb_address 			: in std_logic_vector(31 downto 0);
								signal pix_data 			: in std_logic_vector(7 downto 0);
 								signal mcu_selected_once 	: inout std_logic;
								signal aligned_read_value 	: in std_logic_vector(31 downto 0);
								signal mu_buss_data_out		: out std_logic_vector(31 downto 0);
								signal rp_writer_irq		: out std_logic;
								signal rp_writer_select 		: in std_logic) is
	begin
		if mcu_selected_once = '1' and rp_writer_select = '0' then
			rpw_state <= RPW_COMPLETING;
			rp_writer_irq <= '0';
		elsif rp_writer_select = '1' then
			mcu_selected_once <= '1';
			if mu_buss_address = X"FFC" then
				mu_buss_data_out <=  X"80" & X"000004";
			elsif mu_buss_address = X"FFD" then
				mu_buss_data_out <= fb_address(31 downto 2) & "00";
			elsif mu_buss_address = X"000" then
				case fb_address(1 downto 0) is --HACK thsese are not ordered logically, but the change seemed to improve the line drawing.
				when "00" => mu_buss_data_out <= aligned_read_value(31 downto 8) & pix_data;
				when "01" => mu_buss_data_out <= aligned_read_value(31 downto 16) & pix_data & aligned_read_value(7 downto 0);
				when "10" => mu_buss_data_out <= aligned_read_value(31 downto 24) & pix_data & aligned_read_value(15 downto 0);
				when "11" => mu_buss_data_out <= pix_data & aligned_read_value(23 downto 0);
				when others => null;
				end case;
			end if;
		end if;
	end HandleMCUDrivenWriteClock;

begin


	mu_buss_data_in <= mu_buss_data;
	process(tvc_clock, rp_writer_select, mu_buss_strobe, mu_buss_write, mu_buss_data, mu_buss_data_out)
	begin
		if rp_writer_select	= '1' and mu_buss_strobe = '1' and mu_buss_write = '0' then
			mu_buss_data <= mu_buss_data_out;
		else
			mu_buss_data <= (others => 'Z');
		end if;
	end process;

	process (rpw_state)
	begin
		if rpw_state = RPW_IDLE then
			epp_idle <= '1';
		else
			epp_idle <= '0';
		end if;
	end process;


	Process (tvc_clock)
	begin
	if tvc_clock = '1' and tvc_clock'event then
	if tvc_reset = '1' then
		rpw_state <= RPW_IDLE;
		fb_address <= X"0000_0000";
		pix_data <= X"00";
		aligned_read_value <= X"0000_0000";
		rp_writer_irq <= '0';
	else --not reset
	   case rpw_state is
	   	when RPW_IDLE =>
	 		if epp_start_strobe = '1' then
				fb_address <= epp_fb_address;
				pix_data <= epp_pix_data;
				rp_writer_irq <= '1';
				mcu_selected_once <= '0';
				rpw_state <= RPW_MCU_DRIVEN_READ;
			end if;
		when RPW_MCU_DRIVEN_READ =>
			HandleMCUDrivenReadClock(rpw_state, fb_address, mcu_selected_once, aligned_read_value,
								 mu_buss_data_in, mu_buss_data_out, rp_writer_select);
		when RPW_MCU_DRIVEN_WRITE =>
			HandleMCUDrivenWriteClock(rpw_state, fb_address, pix_data, mcu_selected_once, aligned_read_value,
								 mu_buss_data_out, rp_writer_irq, rp_writer_select);
		when RPW_COMPLETING =>
			 rpw_state <= RPW_IDLE;
		when others => null;
		end case;
	end if; --reset
	end if; --clock
	end process;


end Behavioral;
